/*

  hack.c - Definitions for helper X functions (from rss-glx)

  Copyright (C) 2004 Alastair Tse <acnt2@cam.ac.uk>,
  Copyright (C) 2002 Tugrul Galatali <tugrul@galatali.com>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

  Modified by Alastair Tse to use Python GLHackObject for Python
  compatibility, and other Python related changed.

 */


#include "hack.h"
#include "vroot.h"

int hack_create_window(int argc, char **argv, GLHackObject *self) {
  
	XVisualInfo *visualInfo;
	GLXContext context;

	self->screen_num = DefaultScreen (self->display);
	self->rootWindow = RootWindow (self->display, self->screen_num);

	if (self->usingRootWindow || self->existingWindow) {

	  XWindowAttributes gwa;
	  Visual *visual;
	  XVisualInfo templ;
	  int outCount;

	  self->window = self->existingWindow ? self->existingWindow : self->rootWindow;
	  
	  XGetWindowAttributes (self->display, self->window, &gwa);
	  visual = gwa.visual;
	  self->windowWidth = gwa.width;
	  self->windowHeight = gwa.height;
	  
	  templ.screen = self->screen_num;
	  templ.visualid = XVisualIDFromVisual (visual);
	  
	  visualInfo = XGetVisualInfo (self->display,
								   VisualScreenMask | VisualIDMask, 
								   &templ, &outCount);
	  
	  if (!visualInfo) {
		char errmsg[4096];
		snprintf(errmsg, 4095, "Can't get GL visual for the window 0x%lx.",
				 (unsigned long)self->window);
		PyErr_SetString(PyExc_Exception,  errmsg);
		return -1;
	  }

	} 
	else {
	  int attributeList[] = {
		GLX_RGBA,
		GLX_RED_SIZE, 1,
		GLX_GREEN_SIZE, 1,
		GLX_BLUE_SIZE, 1,
		GLX_DEPTH_SIZE, 1,
		GLX_DOUBLEBUFFER,
		0
	  };
	  XSetWindowAttributes swa;
	  XSizeHints hints;
	  XWMHints wmHints;
	  
	  visualInfo = NULL;

	  if (!(visualInfo = glXChooseVisual (self->display, self->screen_num, attributeList))) {
		char errmsg[4096];
		snprintf(errmsg, 4095, "Can't get GL visual for the window 0x%lx.",
				 (unsigned long)self->window);
		PyErr_SetString(PyExc_Exception,  errmsg);
		return -1;
	  }

	  swa.colormap = XCreateColormap (self->display, self->rootWindow, visualInfo->visual, AllocNone);
	  swa.border_pixel = swa.background_pixel = swa.backing_pixel = BlackPixel (self->display, self->screen_num);
	  swa.event_mask = KeyPressMask | StructureNotifyMask;
	  
	  self->windowWidth = DisplayWidth (self->display, self->screen_num) / 3;
	  self->windowHeight = DisplayHeight (self->display, self->screen_num) / 3;
	  
	  self->window = XCreateWindow (self->display, self->rootWindow, 
									0, 0, 
									self->windowWidth, self->windowHeight, 
									0, visualInfo->depth, 
									InputOutput, visualInfo->visual,
									CWBorderPixel | CWBackPixel | 
									CWBackingPixel | CWColormap | 
									CWEventMask, &swa);

	  hints.flags = USSize;
	  hints.width = self->windowWidth;
	  hints.height = self->windowHeight;
	  
	  wmHints.flags = InputHint;
	  wmHints.input = True;
	  
	  XmbSetWMProperties (self->display, self->window, \
						  self->FIXME_name, self->FIXME_name, argv, argc, \
						  &hints, &wmHints, NULL);
	}
	
	
	context = glXCreateContext (self->display, visualInfo, 0, GL_TRUE);
	if (!context) {
	  PyErr_SetString(PyExc_Exception,  "Can't open GLX Context");
	  return -1;
	}

	if (!glXMakeCurrent (self->display, self->window, context)) {
	  PyErr_SetString(PyExc_Exception,  "Can't open GL Context");
	  return -1;
	}

	XFree (visualInfo);
	XMapWindow (self->display, self->window);
	return 0;
}

void hack_clear_buffers(GLHackObject *self) {
  int i;
  XEvent event;
  
  for (i = 0; i < 4; i++ ) {
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | \
			GL_ACCUM_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);
	glXSwapBuffers (self->display, self->window);
	
	while (XPending (self->display)) {
	  XNextEvent (self->display, &event);
	}
  }
}

void hack_handle_global_opts(GLHackObject *self, int c) {
  switch (c) {
  case 'r':
	self->usingRootWindow = 1;
	break;
  case 'n':
	self->be_nice = 1;
	break;
  case 'x':
	c = strtol_minmaxdef(optarg, 10, 1, 10000, 1, 100, "--maxfps: ");
	self->frameTime = 1000000/c;
	break;
  }
}

int strtol_minmaxdef (char *optarg, int base, int min, int max, \
					  int type, int def, char *errmsg)
{
	int result = strtol (optarg, (char **)NULL, base);

	if (result < min) {
		if (errmsg) {
			fprintf (stderr, errmsg);
			fprintf (stderr, "%d < %d, using %d instead.\n", result, min, type ? min : def);
		}

		return type ? min : def;
	}

	if (result > max) {
		if (errmsg) {
			fprintf (stderr, errmsg);
			fprintf (stderr, "%d > %d, using %d instead.\n", result, max, type ? max : def);
		}

		return type ? max : def;
	}

	return result;
}
